@@ -42,10 +42,15 @@
 #define IS_BIGENDIAN false
 #endif
 
+#include "cross.h"
 #include "drives.h"
 #include "logging.h"
 #include "support.h"
 #include "setup.h"
+
+#if (defined( __MINGW32__) && !defined(__MINGW64__ )) || defined(LINUX)
+#pragma push_macro("__inline__")
+#endif
 #include "src/libs/decoders/audio_convert.c"
 #include "src/libs/decoders/SDL_sound.c"
 #include "src/libs/decoders/vorbis.c"
@@ -54,27 +59,27 @@
 #include "src/libs/decoders/wav.c"
 #include "src/libs/decoders/mp3_seek_table.cpp"
 #include "src/libs/decoders/mp3.cpp"
+#include "src/libs/decoders/dr_flac.h"
+#if (defined( __MINGW32__) && !defined(__MINGW64__ )) || defined(LINUX)
+#pragma pop_macro("__inline__")
+#endif
 #include "src/libs/libchdr/chd.h"
 #include "src/libs/libchdr/libchdr_chd.c"
 #include "src/libs/libchdr/libchdr_cdrom.c"
 #include "src/libs/libchdr/libchdr_flac.c"
 #include "src/libs/libchdr/libchdr_huffman.c"
 #include "src/libs/libchdr/libchdr_bitstream.c"
-#include "src/libs/libchdr/FLAC/stream_decoder.c"
-#include "src/libs/libchdr/FLAC/bitreader.c"
-#include "src/libs/libchdr/FLAC/format.c"
-#include "src/libs/libchdr/FLAC/cpu.c"
-#include "src/libs/libchdr/FLAC/crc.c"
-#include "src/libs/libchdr/FLAC/fixed.c"
-#include "src/libs/libchdr/FLAC/lpc.c"
-#include "src/libs/libchdr/FLAC/md5.c"
-#include "src/libs/libchdr/FLAC/memory.c"
-#if defined(WIN32) && !defined(HX_DOS) && !(defined(__MINGW32__) && !defined(__MINGW64_VERSION_MAJOR))
-#include "src/libs/libchdr/FLAC/windows_unicode_filenames.c"
-#endif
 #include "src/libs/libchdr/lzma/LzmaDec.c"
 #include "src/libs/libchdr/lzma/LzmaEnc.c"
 #include "src/libs/libchdr/lzma/LzFind.c"
+#include "src/libs/libchdr/zstd/common/entropy_common.c"
+#include "src/libs/libchdr/zstd/common/error_private.c"
+#include "src/libs/libchdr/zstd/common/fse_decompress.c"
+#include "src/libs/libchdr/zstd/common/zstd_common.c"
+#include "src/libs/libchdr/zstd/decompress/huf_decompress.c"
+#include "src/libs/libchdr/zstd/decompress/zstd_decompress_block.c"
+#include "src/libs/libchdr/zstd/decompress/zstd_decompress.c"
+#include "src/libs/libchdr/zstd/decompress/zstd_ddict.c"
 
 using namespace std;
 
@@ -509,7 +514,20 @@ bool CDROM_Interface_Image::SetDevice(char* path, int forceCD)
 		uint16_t size = (uint16_t)strlen(buf);
 		DOS_WriteFile(STDOUT, (uint8_t*)buf, &size);
 	}
-	return result;
+
+    int datatracks=0, audiotracks=0;
+    for(const auto& track : tracks) {
+        if(track.attr == 0x40) {
+            datatracks++;
+        }
+        else if(track.attr == 0) {
+            audiotracks++;
+        }
+    }
+    LOG_MSG("CDROM: Image loaded No. of data tracks=%d, audio tracks=%d",
+        datatracks, audiotracks-1
+        );
+    return result;
 }
 
 bool CDROM_Interface_Image::GetUPC(unsigned char& attr, char* upc)
@@ -525,14 +543,15 @@ bool CDROM_Interface_Image::GetAudioTracks(int& stTrack, int& end, TMSF& leadOut
 	end = (int)(tracks.size() - 1);
 	FRAMES_TO_MSF(tracks[tracks.size() - 1].start + 150, &leadOut.min, &leadOut.sec, &leadOut.fr);
 
-	//#ifdef DEBUG
-	LOG_MSG("CDROM: GetAudioTracks, stTrack=%d, end=%d, leadOut.min=%d, leadOut.sec=%d, leadOut.fr=%d",
-	  stTrack,
-	  end,
+	#ifdef DEBUG
+	LOG_MSG("%s CDROM: GetAudioTracks, stTrack=%d, end=%d, leadOut.min=%d, leadOut.sec=%d, leadOut.fr=%d",
+      get_time(),
+      stTrack,
+      end,
 	  leadOut.min,
 	  leadOut.sec,
 	  leadOut.fr);
-	//#endif
+	#endif
 
 	return true;
 }
@@ -566,8 +585,8 @@ bool CDROM_Interface_Image::GetAudioSub(unsigned char& attr, unsigned char& trac
 	track = (unsigned char)cur_track;
 	attr = tracks[track - 1].attr;
 	index = 1;
-	FRAMES_TO_MSF(player.currFrame + 150, &absPos.min, &absPos.sec, &absPos.fr);
-	FRAMES_TO_MSF(player.currFrame - tracks[track - 1].start + 150, &relPos.min, &relPos.sec, &relPos.fr);
+	FRAMES_TO_MSF(player.currFrame, &absPos.min, &absPos.sec, &absPos.fr);
+	FRAMES_TO_MSF(player.currFrame - tracks[track - 1].start, &relPos.min, &relPos.sec, &relPos.fr);
 	if(IS_PC98_ARCH && player.playbackRemaining == 0 && !strcmp(RunningProgram, "ITP")) {
 		// POLICENAUTS
 		// It freeze at the end of the Konami logo or opening.
@@ -582,13 +601,13 @@ bool CDROM_Interface_Image::GetAudioSub(unsigned char& attr, unsigned char& trac
 
 	LOG_MSG("%s CDROM: GetAudioSub absolute offset (%d), MSF=%d:%d:%d",
       get_time(),
-	  player.currFrame + 150,
+	  player.currFrame,
 	  absPos.min,
 	  absPos.sec,
 	  absPos.fr);
 	LOG_MSG("%s CDROM: GetAudioSub relative offset (%d), MSF=%d:%d:%d",
       get_time(),
-	  player.currFrame - tracks[track - 1].start + 150,
+	  player.currFrame - tracks[track - 1].start,
 	  relPos.min,
 	  relPos.sec,
 	  relPos.fr);
@@ -1111,14 +1130,13 @@ bool CDROM_Interface_Image::LoadCloneCDSheet(char *cuefile) {
 	long leadOutLBA = -1;
 	ImageCCDEntry entry;
 	tracks.clear();
-	bool success;
 	int mode=NONE;
 	int shift = 0;
 	int currPregap = 0;
 	int totalPregap = 0;
 	int prestart = -1;
 	int Version=0;
-	int Sessions=0;
+	//int Sessions=0;
 	int TocEntries=0;
 	bool isCloneCD=false;
 	int currentEntry = -1;
@@ -1210,8 +1228,10 @@ bool CDROM_Interface_Image::LoadCloneCDSheet(char *cuefile) {
 				case DISC:
 					if (name == "TocEntries")
 						TocEntries = strtol(value.c_str(),NULL,0);
+					#if 0
 					else if (name == "Sessions")
 						Sessions = strtol(value.c_str(),NULL,0);
+					#endif
 					break;
 				case ENTRY:
 					if (name == "Session")
@@ -1285,12 +1305,13 @@ bool CDROM_Interface_Image::LoadCloneCDSheet(char *cuefile) {
 
 bool CDROM_Interface_Image::LoadCueSheet(char *cuefile)
 {
-	// If we're going to support CUE vs CCD vs anything else then this function must
-	// reject any file who's file extension is not .CUE
+	// reject any file which are not a CUE sheet, GOG is so smart that they set several different extensions so that we can't assume .cue only.
+    // Known extensions at the moment are: .cue, .ins, .dat, .inst (not sure it is an exhaustive list)
 	{
 		char *s = strrchr(cuefile,'.');
 		if (!s) return false;
-		if (strcasecmp(s,".cue")) return false;
+		if (!strcasecmp(s,".ccd") || !strcasecmp(s, ".chd") || !strcasecmp(s, ".iso") || !strcasecmp(s, ".img") || !strcasecmp(s, ".gog")
+            || !strcasecmp(s, ".mds") || !strcasecmp(s, ".mdf") || !strcasecmp(s, ".bin")) return false;
 	}
 
 	Track track = {0, 0, 0, 0, 0, 0, 0, false, NULL};
