@@ -53,6 +53,7 @@ void MSG_Init(), JFONT_Init(), runRescan(const char *str);
 extern int tryconvertcp, toSetCodePage(DOS_Shell *shell, int newCP, int opt);
 extern bool jfont_init;
 extern int msgcodepage;
+extern bool kana_input;
 
 static FILE* OpenDosboxFile(const char* name) {
 	uint8_t drive;
@@ -651,8 +652,14 @@ bool keyboard_layout::map_key(Bitu key, uint16_t layouted_key, bool is_command,
 		}
 
 		// add remapped key to keybuf
-		if (is_keypair) BIOS_AddKeyToBuffer(layouted_key);
-		else BIOS_AddKeyToBuffer((uint16_t)(key<<8) | (layouted_key&0xff));
+        if(is_keypair) {
+            if(kana_input) kana_input = false;
+            else BIOS_AddKeyToBuffer(layouted_key);
+        }
+        else {
+            if(kana_input) kana_input = false;
+            else BIOS_AddKeyToBuffer((uint16_t)(key << 8) | (layouted_key & 0xff));
+        }
 
 		return true;
 	}
@@ -798,7 +805,7 @@ void initcodepagefont() {
 	uint32_t start_pos;
 	uint16_t number_of_codepages;
 	static uint8_t cpi_buff[65536];
-	uint8_t *cpibuf = dos.loaded_codepage == 856 ? cpi_buff : cpi_buf;
+	uint8_t *cpibuf = cpi_buf;
 	uint32_t cpi_buf_size=0,size_of_cpxdata=0;
     switch (dos.loaded_codepage) {
 			case 437:	case 850:	case 852:	case 853:	case 857:	case 858:
@@ -854,7 +861,8 @@ void initcodepagefont() {
 				break;
 			case 856:	case 3846:	case 3848:
 				cpi_buf_size = get_builtin_codepage(bfb_EGA18_CPI);
-				break;
+				cpibuf = cpi_buff;
+                break;
             default:
                     return;
     }
