@@ -35,9 +35,9 @@ class NothingEthernetConnection : public EthernetConnection {
 	public:
 		NothingEthernetConnection();
 		~NothingEthernetConnection();
-		bool Initialize(Section* config);
-		void SendPacket(const uint8_t* packet, int len);
-		void GetPackets(std::function<void(const uint8_t*, int)> callback);
+		bool Initialize(Section* config) override;
+		void SendPacket(const uint8_t* packet, int len) override;
+		void GetPackets(std::function<void(const uint8_t*, int)> callback) override;
 };
 
 #endif
